#include "ADC.h"
#include "IO.h"
#include "I2C.h"
#include "PWM.h"
#include <avr\io.h>
#include <avr/interrupt.h>

/******************************************************************/
/* ADC done INT                                                   */ 
/******************************************************************/
ISR(ADC_vect )
{
 // get values
 unsigned char t8;
 unsigned int  t16;
 
 t8=ADCL;
 t16=ADCH;
 t16=t16 << 8;
 t16+=t8;
 
 ADCFlag|=ADC_Flag_ADCPOTDONE;     // SET flag
 PWM_ADCResult_INT=t16;
 
}
/******************************************************************/
/*                                                                */ 
/******************************************************************/
void ADC_INIT(void)
{
  
  //ADCSRA |=(1<<ADPS2)+(1<<ADPS1)+(1<<ADPS0);    // MCLK/128 = 156 kHz -> fs=12kHz==83s
  //ADCSRA |=(1<<ADPS2)+(1<<ADPS1);             // MCLK/64  = 312 kHz -> fs=24kHz==42s
  
  //ADCSRA |=(1<<ADIE);                           // INT enable
  //ADCSRB &=~(1<<ADLAR);                         // RJ result   
  //ADCSRA |=(1<<ADEN);
  ADCSRA |=(1<<ADPS2)+(1<<ADPS1)+(1<<ADPS0)+(1<<ADIE)+(1<<ADEN);    // MCLK/128 = 156 kHz -> fs=12kHz==83s
  DIDR0  |=(1<<ADC0D);

  
  ADC_MUX_CLR;
  ADC_MUX_POT;


  LPWM_Drivestep=0;
  ADCFlag&=~ADC_Flag_ADCPOTDONE;               // CLR Flag
  ADC_PotvalOLD=0;
  ADCSRA |=(1<<ADSC);                          // Start ADC

}



/******************************************************************/
/*                                                                */ 
/******************************************************************/
void ADC_POT_Task(void)
{
  unsigned char Stepnew;
  unsigned char cnt;
  unsigned int Mean;

  
  /* POT Value */
  if (!(ADCFlag&ADC_Flag_ADCPOTDONE))      return;  // FLAG NOT set ?? -> exit
  if (PWM_Flagreg& PWM_Flag_CTRLI2C)       return;  // ControlMode = I2C ? 

  
  
  
  
  
  
  
  
  ADC_MUX_CLR;
  ADC_MUX_POT;
  ADCFlag&=~ADC_Flag_ADCPOTDONE;               // CLR Flag
  ADCSRA |=(1<<ADSC);                          // Start ADC
  ADC_Potcnt++;                                // inc counter
  
  ADC_Potbuf[ADC_Potcnt]=PWM_ADCResult_INT;    // 1-8
  

  
  
  
  if (ADC_Potcnt>7)                            // accu done ?
  {
  #ifdef MODESWITCHENABLE   
  //*** DEBUG MODE = ON ? ***//
  if ((ADCFlag & ADC_Flag_DEBUG) && (!(PIN_USI & (1<<PIN_USI_SCL))) )
  {
       ADC_MUX_CLR;
       ADC_MUX_MODE; 
       while(!(ADCFlag & ADC_Flag_ADCPOTDONE));     // wait until done
       ADCFlag&=~ADC_Flag_ADCPOTDONE;               // CLR Flag
       ADCSRA |=(1<<ADSC);                          // Start ADC
       while(!(ADCFlag & ADC_Flag_ADCPOTDONE));     // wait until done
       ADCFlag&=~ADC_Flag_ADCPOTDONE;               // CLR Flag
       ADC_ExternalMode();   
       ADC_MUX_CLR;
       ADC_MUX_POT;
       ADCSRA |=(1<<ADSC);                          // Start ADC
  }
  #endif  
    
    
    
    Mean=0;
    for (cnt=1;cnt<9;cnt++)
    {
      Mean+=ADC_Potbuf[cnt];
    }
    
    ADC_Potcnt=0;                             // clr counter
    Mean=Mean >> 3;                           // div by 8

    ADCFlag&=~ADC_Flag_NewValue;
    
    if (Mean>20)
    {
      if ((LPWM_Drivestep==16) || (LPWM_Drivestep==17))
      {
        if ((Mean-35>ADC_PotvalOLD) || (Mean+35<ADC_PotvalOLD))
        {
          ADC_PotvalOLD=Mean;      // more than 2% movement
          ADCFlag|=ADC_Flag_NewValue;
        }
        else
        {
          Mean=ADC_PotvalOLD;      // less than 2% movement
          return;
        }  
      }
      else 
      {
        if ((Mean-20>ADC_PotvalOLD) || (Mean+20<ADC_PotvalOLD))
        {
          ADC_PotvalOLD=Mean;      // more than 2% movement
          ADCFlag|=ADC_Flag_NewValue;
        }
        else
        {
          Mean=ADC_PotvalOLD;      // less than 2% movement
          return;
        }  
      }

    }
    else
    {
      Mean=0;
      ADC_PotvalOLD=Mean;
    }

     
     Stepnew=ADC_Potvale2Drivestep(Mean);       // ADCvalue 2 step
     if (Stepnew==LPWM_Drivestep) return;

     

     LPWM_Drivestep=Stepnew;     
     //PWM_DriveStep2Settings(LPWM_Drivestep);
     PWM_NewDriveStepSettings(LPWM_Drivestep);
  }
  
}

//*******************************************************************//
//*** Poti to Step
//*******************************************************************//
unsigned char ADC_Potvale2Drivestep(unsigned int Potvalue)
{
  unsigned char Step;
  if (Potvalue>=ADC_Potval_Step27_max)  Step=28;
  if (Potvalue<ADC_Potval_Step27_max)  Step=27;
  if (Potvalue<ADC_Potval_Step26_max)  Step=26;
  if (Potvalue<ADC_Potval_Step25_max)  Step=25;
  if (Potvalue<ADC_Potval_Step24_max)  Step=24;
  if (Potvalue<ADC_Potval_Step23_max)  Step=23;
  if (Potvalue<ADC_Potval_Step22_max)  Step=22;
  if (Potvalue<ADC_Potval_Step21_max)  Step=21;
  if (Potvalue<ADC_Potval_Step20_max)  Step=20;
  if (Potvalue<ADC_Potval_Step19_max)  Step=19;
  if (Potvalue<ADC_Potval_Step18_max)  Step=18;
  if (Potvalue<ADC_Potval_Step17_max)  Step=17;
  if (Potvalue<ADC_Potval_Step16_max)  Step=16;
  if (Potvalue<ADC_Potval_Step15_max)  Step=15;
  if (Potvalue<ADC_Potval_Step14_max)  Step=14;
  if (Potvalue<ADC_Potval_Step13_max)  Step=13;
  if (Potvalue<ADC_Potval_Step12_max)  Step=12;
  if (Potvalue<ADC_Potval_Step11_max)  Step=11;
  if (Potvalue<ADC_Potval_Step10_max)  Step=10;
  if (Potvalue<ADC_Potval_Step9_max)  Step=9;
  if (Potvalue<ADC_Potval_Step8_max)  Step=8;
  if (Potvalue<ADC_Potval_Step7_max)  Step=7;
  if (Potvalue<ADC_Potval_Step6_max)  Step=6;
  if (Potvalue<ADC_Potval_Step5_max)  Step=5;
  if (Potvalue<ADC_Potval_Step4_max)  Step=4;
  if (Potvalue<ADC_Potval_Step3_max)  Step=3;
  if (Potvalue<ADC_Potval_Step2_max)  Step=2;
  if (Potvalue<ADC_Potval_Step1_max)  Step=1;
  if (Potvalue<ADC_Potval_Step0_max)  Step=0;

  return(Step);

}


//*******************************************************************//
//*** Setting Ramptimer
//*******************************************************************//
void ADC_ExternalMode(void)
{
 #ifdef MODESWITCHENABLE

    
    /*
    // 0                  = No Mode
    // 146  ->  73..219   = Mode 1
    // 292  -> 220..364   = Mode 2
    // 438  -> 365..511   = Mode 3
    // 584  -> 512..657   = Mode 4
    // 730  -> 658..803   = Mode 5
    // 876  -> 804..940   = Mode 6
    // 1023 -> 941..1024  = Mode 7 
    
    */
    unsigned char oldchar;
    oldchar=LPWM_Charact;
    
    /* 3778 -> 16 Bit */
    if ((PWM_ADCResult_INT > 73)  && (PWM_ADCResult_INT < 219)) LPWM_Charact=1;
    if ((PWM_ADCResult_INT > 220) && (PWM_ADCResult_INT < 364)) LPWM_Charact=2;
    if ((PWM_ADCResult_INT > 365) && (PWM_ADCResult_INT < 511)) LPWM_Charact=3;
    if ((PWM_ADCResult_INT > 512) && (PWM_ADCResult_INT < 657)) LPWM_Charact=4;
    if ((PWM_ADCResult_INT > 658) && (PWM_ADCResult_INT < 803)) LPWM_Charact=5;
    if ((PWM_ADCResult_INT > 804) && (PWM_ADCResult_INT < 940)) LPWM_Charact=6;
    if ((PWM_ADCResult_INT > 940)) LPWM_Charact=7;
    
    if (oldchar !=LPWM_Charact) PWM_DriveStep2Settings(LPWM_Drivestep);
    #endif
}

