  

#include "IO.h"
#include "I2C.h"
#include "PWM.h"
#include "ADC.h"
#include <avr\wdt.h>
#include <avr\io.h>
#include <avr/interrupt.h>


//*********************************************************************//
// Version for PWM with SMPS
//*********************************************************************//

// -added Mode7 (natural PWM)
// -removed USER MODE
// -added WDT Timer Mode
// -added Offset & Offset in EEP
// -added ADC for switch if SCL is low for > 1s at startup
//
// TODO: 
// - Test all the new stuff  
// - rework OVC handling
//
// Rs: U =~0.6V, Ipk ~ 0.8A -> R=0,625R = ~ 1,2R//1,2R
// Mglichkeiten: 1R;1R2;1R5
//
//  - 1R//1R   = 0R5    1.2A @ 0.6V     1.4A  0.7V
//  - 1R//1R2  = 0R55   1.1A @ 0.6V     1.3A  0.7V
//  - 1R//1R5  = 0R6    1.0A @ 0.6V     1.2A  0.7V
//  - 1R2//1R2 = 0R6    1.0A @ 0.6V     1.2A  0.7V
//  - 1R2//1R5 = 0R67   0.9A @ 0.6V     1.0A  0.7V
//  - 1R5//1R5 = 0R75   0.8A @ 0.6V     0.9A  0.7V

/*** MODES ***
// Char. 1 = DC
// Char. 2 = DC
// Char. 3 = PWM1 (125 Hz,  1:1.5)
// Char. 4 = PWM2 (62,5 Hz, 7.5V)
// Char. 5 = PWM3 (62,5 Hz, 1:2)
// Char. 6 = PWM4 (free,    1:4)
// Char. 7 = Pure PWM
/*************/


/* Code to disable wdt in accident case */


void WDTRESETDISABLE(void) \
      __attribute__((naked)) \
      __attribute__((section(".init3")));
    void WDTRESETDISABLE(void)
    {
      MCUSR = 0;
      wdt_disable();
    }


/***** MAIN *****/
//int main(void) __attribute__((noreturn));
int main(void)
{

volatile unsigned char selectchar;


/*** INIT ***/
IO_Init();

PWM_Flagreg|=PWM_Flag_CTRL_ON;
#ifdef MODESWITCHENABLE
asm("sei;");
DEBUG_Test4mechSwitch();         // Test for SCL=Low more than 1 s
asm("cli;");
#endif

ADC_INIT();
PWM_INIT();
#ifdef I2CENABLE
I2C_INIT();
#endif

//*** RUN ***//
asm("sei;");
while(1)
{
  ADC_POT_Task();
  IO_Switch_Task();
  #ifdef I2CENABLE
  I2C_TASK();
  #endif
  PWM_NewDriveStep_Task();
  
    /*
  For debugging purposes !
  */

  selectchar=0;
  asm("nop;");
  /*
  if (selectchar==1)   LPWM_Charact=LPWM_Charact_1;  
  if (selectchar==2)   LPWM_Charact=LPWM_Charact_2;
  if (selectchar==3)   LPWM_Charact=LPWM_Charact_3;
  if (selectchar==4)   LPWM_Charact=LPWM_Charact_4;
  if (selectchar==5)   LPWM_Charact=LPWM_Charact_5;
  if (selectchar==6)   LPWM_Charact=LPWM_Charact_6;
  if (selectchar==7)   LPWM_Charact=LPWM_Charact_7;
  if (selectchar)      PWM_DriveStep2Settings(LPWM_Drivestep);
 */
  if (selectchar)   
  {
    LPWM_Charact=selectchar;
    PWM_DriveStep2Settings(LPWM_Drivestep);
  }  
 
  /*
   \For debugging purposes !
  */

  
  
}
}
/***************************************************************************/
/*
Versions:
V 2.0










*/
