

#include "IO.h"
#include "PWM.h"
#include "I2C.h"
#include "ADC.h"
#include <avr\wdt.h>
#include <avr\io.h>
#include <avr/interrupt.h>

void IO_Init(void)
{
   
    CTRL_SWDIR_INIT;          // Pin = IN
    CTRL_RELONOFF_INIT;       // Pin = OUT
    CTRL_RELONOFF_OFF;        // Output = OFF
    PWM_Flagreg&=~PWM_Flag_CTRL_ON; // set Flag = OFF
    CTRL_OVC_OUT;
    Timer_OVCclear=0;         // clr timer
    
    // INIT OVC-INT
    CTRL_OVC_INIT;      // INIT PIN
    PCMSK0=(1<<PCINT7); // Enable PCINT7
    GIMSK|=(1<<PCIE0);  // Enable PCINT7..0
    
    ADCFlag&=~ADC_Flag_DEBUG;
    
    wdt_enable(WDTO_60MS); // 64ms in TINY24/44/84
    WDTCSR|=(1<<WDIE);     // enable INT instead of reset
    
}

/*******************************************************************************/
/* ISR */
/*******************************************************************************/
ISR(PCINT0_vect)
{
  unsigned char t16;
  
  asm("nop;");
  asm("nop;");
  asm("nop;");
  asm("nop;");
  if (!CTRL_OVC_TST)    // OVC = LO ?
  {
    t16=PWM_OVCcounter; // counting OVC events
    t16++;
    t16++;
    //CTRL_OVC_LEDON;
    if (t16>PWM_OVCcounter_cnt_critical) 
    {
        Timer_OVCclear=PWM_OVCcounter_time_crit;
        CTRL_SWONOFF_INIT;        // Pin = OUT
        CTRL_RELONOFF_OFF;        // Output = OFF
    } 
    //LPWM_PIN_LO;
    OCR0A=5;            // set voltage to ~ 0
    PWM_OVCcounter=t16;
  }


}

/*******************************************************************************/

/*******************************************************************************/
void IO_Switch_Task(void)
{
   unsigned int tFlag;
   
  /* if (!Timer_OVCclear)
   {
        Timer_OVCclear=PWM_OVCtimer_CLR;    // Reload Timer
        t16=PWM_OVCcounter;
        if (t16)    t16--;
        if (!t16)   
        {
            CTRL_OVC_LEDOFF;
            if (PWM_OVCcounter) PWM_OVCcounter--;
        }    
        PWM_OVCcounter=t16; 
   }
   
    */
   
   if (CTRL_SW_COUNTER) return;                 // exit if time not gone....
   CTRL_SW_COUNTER=3;                           // ~ 100ms

   
   
   /*** Control mode = I2C ***/
   tFlag=PWM_Flagreg;
   if (tFlag & PWM_Flag_CTRLI2C)                // ControlMode = I2C ?
   {
        CTRL_RELDIR_INIT;                       // set to output
        if (tFlag & PWM_Flag_CTRL_DIR)    
        {
            CTRL_RELDIR_ON;
        }
        else
        {
            CTRL_RELDIR_OFF;
        }
        
        /***** ONOFF Switch *****/
        /*if (tFlag&PWM_Flag_CTRL_ON)
        {
               CTRL_RELONOFF_ON;
        }
        else
        {
               CTRL_RELONOFF_OFF;
   
        }*/
        return;
   }
   else
   {
        if (LPWM_Drivestep)
        {
       //   CTRL_SWONOFF_Input;                   // set to input
        }
        CTRL_SWDIR_Input;                       // set to input
   }



   /*** Control mode = manual ***/
   asm("cli;");
   tFlag=PWM_Flagreg;
    
   /***** DIR Switch *****/
   if (CTRL_SWDIR_TST)
   {
      tFlag|=PWM_Flag_CTRL_DIR;
   }
   else
   {
      tFlag&=~PWM_Flag_CTRL_DIR;
   }
    
   /***** ONOFF Switch *****/
   if (CTRL_SWONOFF_TST)
   {
        tFlag|=PWM_Flag_CTRL_ON;
   }
   else
   {
        tFlag&=~PWM_Flag_CTRL_ON;  
   
   }
   PWM_Flagreg=tFlag; 
   asm("sei");
   
  
}
/*************************************************************************/ 
/*
unsigned char EEP_Readbyte(unsigned char Adress)
{
  while(EECR&(1<<EEPE));        // wait while previous done
  EEARH=0;
  EEARL=Adress;
  EECR|=(1<<EERE);              // start read
  return(EEDR);
}
*/ 
void EEP_Writebyte(unsigned char Adress, unsigned char DATA)
{
  while(EECR&(1<<EEPE));        // wait while previous done
  EECR=(0<<EEPM1)|(0<<EEPM0);   // set write
  EEARH=0;
  EEARL=Adress;
  EEDR=DATA;
  EECR|=(1<<EEMPE);             
  EECR|=(1<<EEPE);              // start write
  
} 
/*************************************************************************/
void DEBUG_Test4mechSwitch(void)
{
    // for the Debug Mode, the SCL line must be held lo min. 125 cycles at 8ms = 1s
    // ? // this mode is leaved after a reset only. Once entered, no more I2C, if a valid I2C command arrives
    #ifdef MODESWITCHENABLE
    unsigned char exit;
    
    exit=0;
    CTRL_SW_COUNTER=15;         // runs with 12Hz
    while(!exit)
    {
        if (!CTRL_SW_COUNTER)   
        {   
            exit=1;     // Timer done
            ADCFlag|=ADC_Flag_DEBUG;
        }            
        if (PIN_USI & (1<<PIN_USI_SCL)) 
        {
            
            exit=2;     // Pin reaches 1 
            ADCFlag&=~ADC_Flag_DEBUG;
        }
    }
    #endif
}


/*************************************************************************/ 
//* Watchdog INT for use as a timer *//
//* 64ms = 15,6Hz in TINY24/44/84
ISR(WDT_vect)
{
  WDTCSR|=(1<<WDIE);     // enable INT instead of reset
  if (CTRL_SW_COUNTER) CTRL_SW_COUNTER--;
  if  (Timer_OVCclear)  Timer_OVCclear--; 
}

/*************************************************************************/
// Enter Debug/Switch Mode: hold SCL=LO at startup
// Step: 0 (0V) = No Mode / keep Old one
// Step: 1      = Mode 1
// ...
// Step: 7 (5V) = Mode 7
// ==> 7 equal resistors, 8 steps
/*************************************************************************/

