#include "PWM.h"
#include "IO.h"
#include "ADC.h"
#include <avr\io.h>
#include <avr/interrupt.h>



/******************************************************************/
/* Init PWM                                                       */ 
/******************************************************************/
void PWM_INIT(void)
{
  PWM_Flagreg&=~(PWM_Flag_START+PWM_Flag_Rangier);
  
//  LPWM_USER_Mode=LPWM_USER_Mode_internal;  
  PWM_RUNFlagreg&=~PWM_RUNFlag_PWMEOC;
  
   
  // Set default values for PWM (=OFF)

  OCR0A=3;  
  // Timer 0 = VPWM in FastPWM Mode (3-> WGM01=WGM00=1, WGM02=0)
    
  LPWM_PIN_LO;                                // CLR OC0A Port
  LPWM_PIN_OUT;                               // OC0A = OUT-VPWM
  TCCR0A|=(1<<WGM01)+(1<<WGM00)+(1<<COM0A1);  // Mode 3 (1), OC0A clr on BOT, set on match
  //TCCR0B&=~(1<<WGM02);                        // Mode 3 (2) Bit is 0 by default !
  TCCR0B|=(1<<CS00);                          // MCLK / 1 = 62,5kHz
  
  // Timer 1 = LPWM in FastPWM (14-> WGM13=WGM12=WGM11=1, WGM10=0, ICR1=is period)
  


  //TIMSK1=(1<<OCIE1A)+(1<<OCIE1B)+(1<<TOIE1);              // enable OCR1A+B & TOVFL INTs
  //TIMSK1=(1<<OCIE1A)+(1<<OCIE1B)+(1<<ICIE1);
  TIMSK1=(1<<OCIE1A)+(1<<ICIE1);
  //TIMSK1=(1<<OCIE1A)+(1<<TOIE1);                            // enable OCR1A & TOVFL INTs
  //TCCR1A=((1<<WGM11));                                      // Mode 14
  //TCCR1B=((1<<WGM12)+(1<<WGM13)+(1<<CS11));               // Mode 14, MCLK/8   + Start Timer
  TCCR1B=((1<<WGM12)+(1<<WGM13));                           // Mode 14, MCLK/8   
  // mode 12

  while(EECR&(1<<EEPE));        // wait while previous done
  EEARH=0;
  EEARL=EEP_Adress_OFFSET;
  EECR|=(1<<EERE);              // start read
  LPWM_Offset=EEDR;
  if (LPWM_Offset> 0x40) LPWM_Offset=0;
  
  
  VPWM_PIN_OUT;                              // enable VPWM
  LPWM_Charact=LPWM_Charact_1;               // default
  PWM_DriveStep2Settings(0);
  
}


/******************************************************************/
/*  INT - Timer0 OVFL                                             */ 
/******************************************************************/
#ifndef MODE_Timer0_NoOVFL
/*ISR(TIM0_OVF_vect)
{
  
}*/
#endif

/******************************************************************/
/*  INT - Timer1 OCB -> OFF PWM 80/250Hz                          */ 
/******************************************************************/

//ISR(TIM1_COMPB_vect) __attribute__ ((naked));
/*ISR(TIM1_COMPB_vect)
{
  
}*/
/******************************************************************/
/*  INT - Timer1 OCA -> */ 
/******************************************************************/
ISR(TIM1_COMPA_vect)
{
  volatile unsigned int t16;
   
  /*** END-OF-CYCLE flag ***/
  if (PWM_RUNFlagreg & PWM_RUNFlag_PWMEOC)
  {
      PWM_RUNFlagreg&=~PWM_RUNFlag_PWMEOC;      // clr flag
      TCCR1B&=~((1<<CS11)+(1<<CS10));           // STOP
      return;
  } 
  
  
  /*** Pure PWM ***/
  if ((LPWM_Charact==LPWM_Charact_7) && ((LPWM_Drivestep<17)))
  {
    // Pure PWM
    LPWM_PIN_HI;             // Lo Voltage
    PWM_RUNFlagreg|=(PWM_RUNFlag_PWMON+PWM_RUNFlag_PWMEND);
    if (OCR0A < PWM_VPWM_DC)
        {
          OCR0A++;                            // INC Voltage
        }
        else
        {
          OCR0A--;
        }    
    //return;
  }
  
  //*** add. PWM ***/ 
  if (PWM_RUNFlagreg & PWM_RUNFlag_PWMON)
  {
    //**** PWM MODE = ON ****//
    
    /* PWM max. reached */
    if (!(PWM_RUNFlagreg & PWM_RUNFlag_PWMMAX))
    {
        // not max -> rising
        if (OCR0A < PWM_VPWM_PEAK)
        {
            OCR0A++;                            // INC Voltage
        }
        else
        {
          PWM_RUNFlagreg|=PWM_RUNFlag_PWMMAX;   // max reached
        }    
    } 
    
    /* PWM MAX/END */
    if ((PWM_RUNFlagreg & PWM_RUNFlag_PWMMAX) && (!(PWM_RUNFlagreg & PWM_RUNFlag_PWMEND)))
    {
        // max & !end -> falling
        if (OCR0A > PWM_VPWM_DC)
        {
            OCR0A-=2;                            // DEC Voltage
        }
        else
        {
          PWM_RUNFlagreg|=PWM_RUNFlag_PWMEND;   // end reached
        }    
    }


    /* next OCR event */
    if (!(PWM_RUNFlagreg & PWM_RUNFlag_PWMEND))
    {
        // !end ? -> calc next OCRA event
        t16=OCR1AH;
        t16=t16 << 8;
        t16+=OCR1AL;
        t16+=LPWM_OCR1A_Stepsize;
        OCR1AH=(t16 >> 8) & 0xFF;
        OCR1AL=(t16 ) & 0xFF;
    }  
    else
    {
        // end & mode = PWM4 ?
        if (LPWM_Charact==LPWM_Charact_6)
        {
 
            TCCR1B&=~((1<<CS11)+(1<<CS10));              // STOP
            TCNT1H=LPWM_Width_FREECH;                    // set to almost end
            TCNT1L=LPWM_Width_FREECL;  
            TCCR1B|=(1<<CS11);                           // MCLK / 8      
        }
    }  
  }
  else
  {
        //**** PWM MODE = OFF *****//
        // not max -> rising
        if (OCR0A < PWM_VPWM_DC)
        {
          OCR0A++;                            // INC Voltage
        }
        else
        {
          OCR0A--;
        }    
          // !end ? -> calc next OCRA event
        t16=OCR1AH;
        t16=t16 << 8;
        t16+=OCR1AL;
        t16+=LPWM_OCR1A_Stepsize;
        OCR1AH=(t16 >> 8) & 0xFF;
        OCR1AL=(t16 ) & 0xFF;
  }

  if (!CTRL_OVC_TST)    // OVC = LO ?
  {
    CTRL_OVC_LEDON;

  }
}
// Char. 1 = DC
// Char. 2 = DC
// Char. 3 = PWM1 (125 Hz,  1:1.5)
// Char. 4 = PWM2 (62,5 Hz, 7.5V)
// Char. 5 = PWM3 (62,5 Hz, 1:2)
// Char. 6 = PWM4 (free,    1:4)
// Char. 7 = Pure PWM

/******************************************************************/
/*  INT - Timer1 OVFL -> Start PWM                      */ 
/******************************************************************/
//ISR(TIM1_OVF_vect)
ISR(TIM1_CAPT_vect)
{

    /*** Overcurrent Timer gone ***/
    if (PWM_OVCcounter<PWM_OVCcounter_cnt_critical)    
      {
        CTRL_OVC_LEDOFF;
      }  
    
    /*** Mode independent Output FET ON ***/
    // to be changed if NFET is used !
    LPWM_PIN_LO;  // Hi Voltage
    // to be changed if NFET is used !
    
    
    if (PWM_OVCcounter) PWM_OVCcounter--;
    
    PWM_RUNFlagreg&=~(PWM_RUNFlag_PWMEND+PWM_RUNFlag_PWMMAX+PWM_RUNFlag_PWMEOC);   // clr end+max flag

    
    /*** Mode dependent OCR1A settings ***/
    if ((LPWM_Charact==LPWM_Charact_7) && (LPWM_Drivestep<17))
    {
        // MODE = 7 / PWM
        //LPWM_Drivestep ->   // 1..16 -> 10..100%
                              // Timer = 125Hz -> 16000==100%
        
        OCR1AH=(LPWM_Drivestep-1) << 2;
        OCR1AL=0;
    }
    else
    {
        // MODE <> 7 
        // if  LPWM_OCR1A_Stepsize < 255 !
        OCR1AH=0;
        OCR1AL=LPWM_OCR1A_Stepsize;
    }
    
    
    //**** ON/OFF Relais behaviour ****//
    
    if (!LPWM_Drivestep)          // Step=0 --> Output = OFF   
    {
        // Step = 0 -> OFF
        CTRL_RELONOFF_INIT;        // Pin = OUT
        CTRL_RELONOFF_OFF;        // Output = OFF
    }    
    else                          // Step > 0
    {
        // Step > 0 ->
        if (PWM_Flagreg & PWM_Flag_CTRLI2C)   // ControlMode = I2C ?
        {
         // MODE = I2C
            if (PWM_Flagreg& PWM_Flag_CTRL_ON)// Flag = ON ?
            {
               // Flag = ON
               if (!Timer_OVCclear)           // OVC Timer runs ? Y-> not ON !
               {
                CTRL_SWONOFF_INIT;            // Pin = OUT
                CTRL_RELONOFF_ON;             // Output = ON
               } 
            }
            else
            {
               // Flag = OFF
               CTRL_RELONOFF_INIT;             // Pin = OUT
               CTRL_RELONOFF_OFF;              // Output = OFF
           }
        }
        else
        {                             
          // MODE = NON-I2C = Manual
          if (!Timer_OVCclear)                // OVC Timer runs ? Y-> not ON !
          {
            CTRL_SWONOFF_INIT;                // set to input
          } 
        }
   }



    
    /* uses wdt instead !
    
    // Timer (SUB) runs with   25 Hz

    if (LPWM_TimerSub<CTRL_SW_COUNTER) 
        {
          CTRL_SW_COUNTER-=LPWM_TimerSub;
        }
        else
        {
          CTRL_SW_COUNTER=0;
        }    

    if (LPWM_TimerSub<Timer_OVCclear) 
        {
          Timer_OVCclear-=LPWM_TimerSub;
        }
        else
        {
          Timer_OVCclear=0;
        }    

      */
    
}

/******************************************************************/
/*                                                                */ 
/******************************************************************/
void PWM_NewDriveStepSettings(unsigned char Step_lokal)
{
   LPWM_DrivestepNEW=Step_lokal;        // store new Step
   PWM_RUNFlagreg|=(PWM_RUNFlag_PWMEOC+PWM_RUNFlag_PWMNEW);
}
void PWM_NewDriveStep_Task(void)
{
    if (PWM_RUNFlagreg & PWM_RUNFlag_PWMNEW)
    {
       if (!(PWM_RUNFlagreg&PWM_RUNFlag_PWMEOC))    // flag was cleared by timer, timer stopped
       {
         PWM_DriveStep2Settings(LPWM_DrivestepNEW);
         PWM_RUNFlagreg&=~PWM_RUNFlag_PWMNEW;
       }
    }
}
/******************************************************************/
/*                                                                */ 
/******************************************************************/
void PWM_DriveStep2Settings(unsigned char Step_lokal)
{
  //unsigned char t161,t162;
  if (!Step_lokal)
  {
    // no output, step = 0 !
    ICR1H=LPWM_Width_62H;    // Timer End
    ICR1L=LPWM_Width_62L;

    OCR0A=0;
    LPWM_PIN_HI;             // Lo Voltage
    CTRL_RELONOFF_INIT;      // Pin = OUT
    CTRL_RELONOFF_OFF;       // Output = OFF
    TCCR1B|=(1<<CS11);       // MCLK / 8
    return;
  }

// Char. 1 = DC
// Char. 2 = DC
// Char. 3 = PWM1 (125 Hz,  1:1.5)
// Char. 4 = PWM2 (62,5 Hz, 7.5V)
// Char. 5 = PWM3 (62,5 Hz, 1:2)
// Char. 6 = PWM4 (free,    1:4)
// Char. 7 = Pure PWM

  if (Step_lokal > 16)
  {
    // 2nd Level, no PWM anymore
    Step_lokal-=17;
    PWM_RUNFlagreg&=~PWM_RUNFlag_PWMON;
    switch(LPWM_Charact)
    {
      default:    break;
      case LPWM_Charact_7:
      case LPWM_Charact_1: 
                  // 8 LSB
                  PWM_VPWM_DC=Step_lokal << 3;
                  PWM_VPWM_DC+=LPWM_Char1_STARTLSBL2;
                  //wdt LPWM_TimerSub=5;                      // Timer runs with 25 Hz
                  break;
      case LPWM_Charact_2: 
                  // 6 LSB
                  PWM_VPWM_DC=Step_lokal << 2;
                  PWM_VPWM_DC+=Step_lokal << 1;
                  PWM_VPWM_DC+=LPWM_Char2_STARTLSBL2;
                  //wdt LPWM_TimerSub=5;                      // Timer runs with 25 Hz
                  break;
      case LPWM_Charact_3: 
                  // 10 LSB
                  PWM_VPWM_DC=Step_lokal << 3;
                  PWM_VPWM_DC+=Step_lokal << 1;
                  PWM_VPWM_DC+=LPWM_Char3_STARTLSBL2;
                  //wdt LPWM_TimerSub=5;                      // Timer runs with 25 Hz
                  break;
      case LPWM_Charact_4: 
                  // 8 LSB
                  PWM_VPWM_DC=Step_lokal << 3;
                  PWM_VPWM_DC+=LPWM_Char4_STARTLSBL2;
                  //wdt LPWM_TimerSub=3;                      // Timer runs with 25 Hz
                  break;   
      case LPWM_Charact_5: 
                  // 10 LSB
                  PWM_VPWM_DC=Step_lokal << 3;
                  PWM_VPWM_DC+=Step_lokal << 1;
                  PWM_VPWM_DC+=LPWM_Char5_STARTLSBL2;
                  //wdt LPWM_TimerSub=3;                      // Timer runs with 25 Hz
                  break;   
      case LPWM_Charact_6: 
                  // 10 LSB
                  PWM_VPWM_DC=Step_lokal << 3;
                  PWM_VPWM_DC+=Step_lokal << 1;
                  PWM_VPWM_DC+=LPWM_Char6_STARTLSBL2;
                  //wdt LPWM_TimerSub=5;                      // Timer runs with 25 Hz
                  break;   
  /*    case LPWM_Charact_7:                              // natural PWM !!! 
                  PWM_VPWM_DC=123;
                  break;  
    */}               

      Step_lokal+=17;
  }
  else
  {
   // 1st Level, DC+PWM 
   Step_lokal--;
   PWM_RUNFlagreg|=PWM_RUNFlag_PWMON;
   switch(LPWM_Charact)
    {
      default:    break;
      case LPWM_Charact_1:
                  PWM_RUNFlagreg&=~PWM_RUNFlag_PWMON;
                  // 6 LSB
                  PWM_VPWM_DC=Step_lokal << 2;
                  PWM_VPWM_DC+=Step_lokal << 1;
                  PWM_VPWM_DC+=LPWM_Char1_STARTLSBL1;
                  //wdt LPWM_TimerSub=5;                      // Timer runs with 25 Hz
                  break;
      case LPWM_Charact_2: 
                  PWM_RUNFlagreg&=~PWM_RUNFlag_PWMON;
                  // 4 LSB
                  PWM_VPWM_DC=Step_lokal << 2;
                  PWM_VPWM_DC+=LPWM_Char2_STARTLSBL1;
                  //wdt LPWM_TimerSub=5;                      // Timer runs with 25 Hz
                  break;
      case LPWM_Charact_3: 
                  // 4 LSB
                  PWM_VPWM_DC=Step_lokal << 2;
                  PWM_VPWM_DC+=LPWM_Char3_STARTLSBL1;
                  PWM_VPWM_PEAK=PWM_VPWM_DC;            // 1x...
                  PWM_VPWM_PEAK+=PWM_VPWM_DC >> 1;      // + 0.5x
                  //wdt LPWM_TimerSub=5;                      // Timer runs with 25 Hz
                  break;
      case LPWM_Charact_4: 
                  // 6 LSB
                  PWM_VPWM_DC=Step_lokal << 2;
                  PWM_VPWM_DC+=Step_lokal << 1;
                  PWM_VPWM_DC+=LPWM_Char4_STARTLSBL1;
                  PWM_VPWM_PEAK=LPWM_Char4_VPWM ;       // fixed 
                  //wdt LPWM_TimerSub=3;                      // Timer runs with 25 Hz
                  break;   
     case LPWM_Charact_5: 
                  // 4 LSB
                  PWM_VPWM_DC=Step_lokal << 2;
                  PWM_VPWM_DC+=LPWM_Char5_STARTLSBL1;
                  PWM_VPWM_PEAK=PWM_VPWM_DC << 1;       // x2 
                  //wdt LPWM_TimerSub=3;                      // Timer runs with 25 Hz
                  break;   
      case LPWM_Charact_6: 
                  // 1 LSB
                  PWM_VPWM_DC=Step_lokal;
                  PWM_VPWM_DC+=LPWM_Char6_STARTLSBL1;
                  PWM_VPWM_PEAK=PWM_VPWM_DC << 1;       // x2 
                  PWM_VPWM_PEAK+=PWM_VPWM_DC ;          // +1

                  //wdt Timer
                  
                  /*
                    if (Step_lokal==0)  LPWM_TimerSub=5;
                    if ((Step_lokal>0) && (Step_lokal<3))   LPWM_TimerSub=12;
                    if (Step_lokal==3)  LPWM_TimerSub=10;
                    if (Step_lokal==4)  LPWM_TimerSub=9;
                    if (Step_lokal==5)  LPWM_TimerSub=8;
                    if ((Step_lokal>5) && (Step_lokal<8))   LPWM_TimerSub=7;
                    if ((Step_lokal>7) && (Step_lokal<10))  LPWM_TimerSub=6;
                    if ((Step_lokal>9) && (Step_lokal<14))  LPWM_TimerSub=5;
                    if ((Step_lokal>13) && (Step_lokal<17)) LPWM_TimerSub=4;
                    */  
                  break;   
      case LPWM_Charact_7:                              // natural PWM !!! 
                  PWM_VPWM_DC=LPWM_Char4_VPWM ;         // fixed 
                  break;  
                  
    } 
    Step_lokal++;          
  }
    
  if ((0xFF-PWM_VPWM_DC)>LPWM_Offset)   PWM_VPWM_DC+=LPWM_Offset; 
  if ((0xFF-PWM_VPWM_PEAK)>LPWM_Offset) PWM_VPWM_PEAK+=LPWM_Offset;

  
// Char. 1 = DC
// Char. 2 = DC
// Char. 3 = PWM1 (125 Hz,  1:1.5)
// Char. 4 = PWM2 (62,5 Hz, 7.5V)
// Char. 5 = PWM3 (62,5 Hz, 1:2)
// Char. 6 = PWM4 (free,    1:4)
// Char. 7 = Pure PWM

  switch(LPWM_Charact)
  {
    default:
              break;
 
    case      LPWM_Charact_7: 
    case      LPWM_Charact_3:           // PWM (125Hz)
    case      LPWM_Charact_1:           // DC
    case      LPWM_Charact_2:           // DC    
              ICR1H=LPWM_Width_125H;    // Timer End
              ICR1L=LPWM_Width_125L;
              TCNT1H=LPWM_Width_125CH;
              TCNT1L=LPWM_Width_125CL;
              break;                       
    
    case      LPWM_Charact_4:           // PWM (62,5Hz)
    case      LPWM_Charact_5:           // PWM (62,5Hz)
              ICR1H=LPWM_Width_62H;     // Timer End
              ICR1L=LPWM_Width_62L;
              TCNT1H=LPWM_Width_62CH;
              TCNT1L=LPWM_Width_62CL;
              break;                       
  
    case      LPWM_Charact_6:           // PWM "free"
              if (Step_lokal>16)
              {
                  ICR1H=LPWM_Width_125H;    // Timer End
                  ICR1L=LPWM_Width_125L;
                  TCNT1H=LPWM_Width_125CH;
                  TCNT1L=LPWM_Width_125CL;
              }
              else
              {
                  ICR1H=LPWM_Width_FREEH;    // Timer End
                  ICR1L=LPWM_Width_FREEL; 
                  TCNT1H=LPWM_Width_FREECH;
                  TCNT1L=LPWM_Width_FREECL;
             
              }        
              break;                       
    
    /*case      LPWM_Charact_7:                     // natural PWM !!! 
              break;  */

  }
  
  
  
  /* old USER settings stuff here 
  if (LPWM_USER_Mode==LPWM_USER_Mode_table)
  {
     PWM_VPWM_DC=LPWM_USER_Voltage[Step_lokal];  
     switch(LPWM_Charact)
     {
        default:     break;
        case      LPWM_Charact_1:           // DC
        case      LPWM_Charact_2:           // DC    
                  break;                       
        case      LPWM_Charact_4:           // PWM (125Hz) / x2
                  if ((Step_lokal < 17) && (PWM_VPWM_DC>0x7F)) PWM_VPWM_DC=0x7F;
                  PWM_VPWM_PEAK+=PWM_VPWM_DC << 1;          // x2
                  break;
        case      LPWM_Charact_3:           // PWM / x1,5
                  if ((Step_lokal < 17) &&(PWM_VPWM_DC>169))    PWM_VPWM_DC=169;
                  PWM_VPWM_PEAK=PWM_VPWM_DC;                // 1x...
                  PWM_VPWM_PEAK+=PWM_VPWM_DC >> 1;          // + 0.5x
                  break;                       
        case      LPWM_Charact_5:           // PWM / x4
                  if ((Step_lokal < 17) &&(PWM_VPWM_DC>63))     PWM_VPWM_DC=63;
                  PWM_VPWM_PEAK+=PWM_VPWM_DC << 4;          // x4
                  break;                       
     }           
  }*/


 /*** Rangiergang ***/  
 if (PWM_Flagreg & PWM_Flag_Rangier)
 {
    PWM_VPWM_PEAK = PWM_VPWM_PEAK >> 1; 
    PWM_VPWM_DC = PWM_VPWM_DC >> 1;
 }
  TCCR1B|=(1<<CS11);         // MCLK / 8
  
}



/******************************************************************/
/*                                                                */ 
/******************************************************************/

/******************************************************************/
/*                                                                */ 
/******************************************************************/




