#ifndef PWM_H
#define PWM_H


/****************************************************************/
// CPU = TINY44
// MCLK = 16MHz
/****************************************************************/

/*******************************************************************************/
//
// VPWM  is connected to OC0A / PB2 / PIN-5
// LPWM  is connected to OC1B / PA5 / PIN-8
//
/*******************************************************************************/

#define LPWM_PIN_OUT   DDRA|=(1<<5)
#define LPWM_PIN_IN    DDRA&=~(1<<5)
#define LPWM_PIN_HI    PORTA|=(1<<5)
#define LPWM_PIN_LO    PORTA&=~(1<<5)

#define VPWM_PIN_OUT   DDRB|=(1<<2)


/*******************************************************************************/

/*******************************************************************************/

/**** Set LPWM frequency here ! ****/
// 62Hz -> 32000
#define LPWM_Width_62H     0x7D 
#define LPWM_Width_62L     0x00
#define LPWM_Width_62CH    0x7C 
#define LPWM_Width_62CL    0xBF

// 125Hz -> 16000
#define LPWM_Width_125H     0x3E 
#define LPWM_Width_125L     0x80
#define LPWM_Width_125CH    0x3E 
#define LPWM_Width_125CL    0x5B

// Free -> 65000
#define LPWM_Width_FREEH     0xFF 
#define LPWM_Width_FREEL     0xFE
#define LPWM_Width_FREECH    0xFF 
#define LPWM_Width_FREECL    0xBF

/**** Set LPWM frequency here END ****/

#define LPWM_OCR1A_Stepsize 80
#define LPWM_Char4_VPWM     130

/**********************************************************************************************/

#define LPWM_Charact_1  1
#define LPWM_Charact_2  2
#define LPWM_Charact_3  3
#define LPWM_Charact_4  4
#define LPWM_Charact_5  5
#define LPWM_Charact_6  6
#define LPWM_Charact_7  7
// Char. 1 = DC
// Char. 2 = DC
// Char. 3 = PWM1 (125 Hz,  1:1.5)
// Char. 4 = PWM2 (62,5 Hz, 7.5V)
// Char. 5 = PWM3 (62,5 Hz, 1:2)
// Char. 6 = PWM4 (free,    1:4)
// Char. 7 = Pure PWM

// Table #5
#define LPWM_Char1_STARTLSBL1 36
#define LPWM_Char2_STARTLSBL1 36
#define LPWM_Char3_STARTLSBL1 35
#define LPWM_Char4_STARTLSBL1 35
#define LPWM_Char5_STARTLSBL1 35 
#define LPWM_Char6_STARTLSBL1 35 

#define LPWM_Char1_STARTLSBL2 136
#define LPWM_Char2_STARTLSBL2 102
#define LPWM_Char3_STARTLSBL2 107
#define LPWM_Char4_STARTLSBL2 129
#define LPWM_Char5_STARTLSBL2 117
#define LPWM_Char6_STARTLSBL2 112

/**********************************************************************************************/

#define LPWM_USER_Mode_internal     0
#define LPWM_USER_Mode_table        1

/**********************************************************************************************/
                                                                                                
#define PWM_Flag_LIGHT    (1<<1)
#define PWM_Flag_CTRLI2C  (1<<2)
#define PWM_Flag_CTRL_DIR (1<<3)
#define PWM_Flag_CTRL_ON  (1<<4)
#define PWM_Flag_Rangier  (1<<5)
#define PWM_Flag_START    (1<<6)

#define PWM_RUNFlag_PWMEND   (1<<0)
#define PWM_RUNFlag_PWMMAX   (1<<1)
#define PWM_RUNFlag_PWMON    (1<<2)
#define PWM_RUNFlag_PWMEOC   (1<<3)
#define PWM_RUNFlag_PWMNEW   (1<<4)

//#define PWM_OVCtimer_CLR                50
#define PWM_OVCcounter_cnt_critical     25
#define PWM_OVCcounter_time_crit        2500

/**********************************************************************************************/


/**********************************************************************************************/

/**********************************************************************************************/
/* !!!! DON't foregt to activate DEBUG flag for debugging parameters !!!! */





/* !!!! DON't foregt to activate DEBUG flag for debugging parameters !!!! */
/**********************************************************************************************/
volatile unsigned char  PWM_VPWM_DC;            // DutyCycle for VoltagePWM, goal value
volatile unsigned char  PWM_VPWM_PEAK;          // DutyCycle for VoltagePWM, goal value
//volatile unsigned int   LPWM_TimerSub;          // PWM Frequency Range
volatile unsigned int   LPWM_Charact;           // Chayracteristic
volatile unsigned char  LPWM_Drivestep;         // LPWM DriveStep for storage
volatile unsigned char  LPWM_DrivestepNEW;      // LPWM DriveStep for storage
volatile unsigned int   PWM_ADCResult_INT;      // Value of Pot (if used)
volatile unsigned char  PWM_Flagreg; 
volatile unsigned char  PWM_RUNFlagreg;
volatile unsigned char  PWM_OVCcounter;
volatile unsigned int   Timer_OVCclear;
volatile unsigned char  LPWM_Offset;
/**********************************************************************************************/
//volatile unsigned char  LPWM_USER_Voltage[28];
//volatile unsigned char  LPWM_USER_Mode;




/**********************************************************************************************/
//#define DEBUG
#ifdef DEBUG
volatile unsigned char PWM_DBG_I[8];
volatile unsigned char PWM_DBG_I_cnt;

volatile unsigned char PWM_DBG_P[8];
volatile unsigned char PWM_DBG_P_cnt;
#endif

/**********************************************************************************************/

void PWM_INIT(void);
void PWM_DriveStep2Settings(unsigned char Step_lokal);
void PWM_NewDriveStepSettings(unsigned char Step_lokal);
void PWM_NewDriveStep_Task(void);
 
 
 
/**********************************************************************************************/ 
// The Vbe was added here ! Vbe = ~ 660mV -> 10 LSB


/*
2013-10-12 / // Table #4
#define LPWM_Char1_STARTLSBL1 23
#define LPWM_Char2_STARTLSBL1 20
#define LPWM_Char3_STARTLSBL1 39
#define LPWM_Char4_STARTLSBL1 25
#define LPWM_Char5_STARTLSBL1 23 
#define LPWM_Char6_STARTLSBL1 13 

#define LPWM_Char1_STARTLSBL2 123
#define LPWM_Char2_STARTLSBL2 94
#define LPWM_Char3_STARTLSBL2 120
#define LPWM_Char4_STARTLSBL2 142
#define LPWM_Char5_STARTLSBL2 118
#define LPWM_Char6_STARTLSBL2 118
*/



/*
------------------------------------------------------------
------------------------------------------------------------
Timer0: 8-Bit, MCLK = 16MHz
direct OutputCompare
-> MCLK/1 = 16 MHz, TOP=0xFF
-> fpwm = 62,5 kHz
-> 256 Steps




------------------------------------------------------------
Timer1: 16-Bit, MCLK = 16MHz
-> MCLK / 8 = 2MHz ; MCLK / 64 = 250kHz ; MCLK / 256 = 62,5kHz ; MCLK / 1024 = 15,6 kHz

indirect Output compare INT:

 
------------------------------------------------------------
Voltage PWM:
Voutmax = 15V == 100
Voutmax = 15V == 255  -> 58,8mV/Step

*/
#endif
